Package: XAItest
Title: XAItest: Enhancing Feature Discovery with eXplainable AI
Version: 1.4.0
Authors@R: c(
    person("Ghislain", "FIEVET", , "ghislain.fievet@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-0337-7327")),
    person("Sébastien", "HERGALANT", role = "aut",
    email="sebastien.hergalant@inserm.fr", comment=c(ORCID = "0000-0001-8456-7992")))
Description: 
    XAItest is an R Package that identifies features using eXplainable AI (XAI)
    methods such as SHAP or LIME. This package allows users to compare
    these methods with traditional statistical tests like t-tests, empirical
    Bayes, and Fisher's test. Additionally, it includes simThresh, a system
    that enables the comparison of feature importance with p-values
    by incorporating calibrated simulated data.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
biocViews: Software, StatisticalMethod, FeatureExtraction,
        Classification, Regression
Suggests: knitr, ggforce, shapr (>= 1.0.1), airway, xgboost,
        BiocGenerics, RUnit, S4Vectors
Imports: limma, randomForest, kernelshap, caret, lime, DT, methods,
        SummarizedExperiment, ggplot2
VignetteBuilder: knitr
URL: https://github.com/GhislainFievet/XAItest
BugReports: https://github.com/GhislainFievet/XAItest/issues
git_url: https://git.bioconductor.org/packages/XAItest
git_branch: RELEASE_3_23
git_last_commit: a7f2cb4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:28:31 UTC; biocbuild
Author: Ghislain FIEVET [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0337-7327>),
  Sébastien HERGALANT [aut] (ORCID:
    <https://orcid.org/0000-0001-8456-7992>)
Maintainer: Ghislain FIEVET <ghislain.fievet@gmail.com>
Depends: R (>= 3.5.0)
