# Generated by roxygen2: do not edit by hand

export(XAI.test)
export(addSimThresh)
export(genScenario)
export(getFeatImpThresholds)
export(getMetricsTable)
export(mapPvalImportance)
export(modelsOverview)
export(plotModel)
export(setMetricsTable)
exportMethods(show)
importFrom(methods,setMethod)
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,show)
importFrom("methods", "is", "new", "validObject")
importFrom(stats,runif)
importFrom("stats", "as.formula", "cor.test", "lm", "model.matrix",
             "p.adjust", "predict", "qnorm", "rnorm", "t.test")
